      subroutine cg(ld,n,nm,q,p,ww,wi,sigma,tau,temp1,temp2,temp3,
     &              t1,t2,t1i,t2i,q12,u,v,sigm34,sig4,sigm1,sigm4,s1)
*
* Compute a contragredient transformation (W,U,Sigma) of (Q,P).
*
* Input parameters:
* ld - leading dimension of matrices
* n - number of rows in W
* nm -  number of rows in Sigma
* q,p - (Q,P)
*
* Output parameters:
* ww,wi,sigma - transformation (W,U,Sigma), W = U^{-1}
* tau - product of W(1:n,1:nm) and U(1:nm,1:n)
*
* Working storage:
* temp1,temp2,temp3,t1,t2,t1i,t2i,q12,u,v,sigm34,sig4,sigm1,sigm4 -
*      - working matrices
* s1 - working array
*
      integer i,ind,j,ld,n,nm
      double precision p(ld,ld),q(ld,ld),q12(ld,ld),s1(ld),sig4(ld,ld)
      double precision sigm1(ld,ld),sigm34(ld,ld),sigm4(ld,ld)
      double precision sigma(ld,ld),small,t1(ld,ld),t1i(ld,ld),t2(ld,ld)
      double precision t2i(ld,ld),tau(ld,ld),temp1(ld,ld),temp2(ld,ld)
      double precision temp3(ld,ld),tmp,u(ld,ld),v(ld,ld),wi(ld,ld)
      double precision ww(ld,ld)
*
      small = 1.d-3
*
* Compute P = V * diag{Omega 0} * V^t
*
      call rs(ld,n,p,s1,1,v,temp1,temp2,ind)
*
* Compute T1 = V * diag{Omega^{-1/2} I}, T1^{-1} = diag{Omega^{1/2} I} * V^t
*
      call setmat(ld,0.d0,temp1,n,n)
      call setmat(ld,0.d0,temp2,n,n)
*
      do 301 i = 1,nm
        if (s1(n-i+1) .lt. small) s1(n-i+1) = small
        temp1(i,i) = 1.d0 / dsqrt(s1(n-i+1))
        temp2(i,i) = dsqrt(s1(n-i+1))
301   continue
*
      do 302 i = nm+1,n
        temp1(i,i) = 1.d0
        temp2(i,i) = 1.d0
302   continue
*
      do 303 j = 1,n/2
        do 304 i = 1,n
          tmp = v(i,j)
          v(i,j) = v(i,n-j+1)
          v(i,n-j+1) = tmp
304     continue
303   continue
*
      call mulmat(ld,v,temp1,t1,n,n,n,1)
      call mulmat(ld,temp2,v,t1i,n,n,n,3)
*
* Compute T1^{-1} * Q * T1^{-t} and store in temp3,
* store appropriate part of temp3 in Q12
*
      call mulmat(ld,t1i,q,temp1,n,n,n,1)
      call mulmat(ld,temp1,t1i,temp3,n,n,n,3)
*
      call scmat(ld,1.d0,temp3(1,nm+1),q12,nm,n-nm)
*
* Compute Q11 = U * Omega * U^t
*
      call rs(ld,nm,temp3,s1,1,u,temp1,temp2,ind)
*
* Compute Sigma, Omega^{-1}, Omega^{1/4}, Omega^{-1/4}, Omega^{-3/4}
*
      call setmat(ld,0.d0,sigma,nm,nm)
      call setmat(ld,0.d0,sig4,n,n)
      call setmat(ld,0.d0,sigm4,n,n)
      call setmat(ld,0.d0,sigm1,n,n)
      call setmat(ld,0.d0,sigm34,n,n)
      do 311 i = 1,nm
        if (s1(i) .lt. small) s1(i) = small
        sigma(i,i) = dsqrt(s1(i))
        sigm1(i,i) = 1.d0/s1(i)
        sig4(i,i) = dsqrt(dsqrt(s1(i)))
        sigm4(i,i) = 1.d0/sig4(i,i)
        sigm34(i,i) = sigm4(i,i) * sigm4(i,i) * sigm4(i,i)
311   continue
*
* Compute T2, T2^{-1}
*
      call setmat(ld,0.d0,t2,n,n)
      call setmat(ld,0.d0,t2i,n,n)
      do 312 i = nm+1,n
        t2(i,i) = -1.d0
        t2i(i,i) = -1.d0
312   continue
*
* Compute U * Sigma^{1/4}, Sigma^{-1/4} * U^t
*
      call mulmat(ld,u,sig4,t2,nm,nm,nm,1)
      call mulmat(ld,sigm4,u,t2i,nm,nm,nm,3)
*
* Compute Q12^t * U * Sigma^{-3/4}, Q12^t * U * Sigma^{-1} * U^t
*
      call mulmat(ld,q12,u,temp1,n,nm,nm,2)
      call mulmat(ld,temp1,sigm34,temp3,n,nm,nm,1)
*
      call mulmat(ld,q12,u,temp1,n,nm,nm,2)
      call mulmat(ld,temp1,sigm1,temp2,n,nm,nm,1)
      call mulmat(ld,temp2,u,temp1,n,nm,nm,3)
*
      do 321 i = 1,n-nm
        do 322 j = 1,nm
          t2(nm+i,j) = temp3(i,j)
          t2i(nm+i,j) = temp1(i,j)
322     continue
321   continue
*
* Compute W = T1 * T2, U = W^{-1} = T2^{-1} * T1^{-1}, tau = W * W^{-1}
*
      call mulmat(ld,t1,t2,ww,n,n,n,1)
      call mulmat(ld,t2i,t1i,wi,n,n,n,1)
      call mulmat(ld,ww,wi,tau,n,n,nm,1)
*
      return
      end
