      subroutine initp2(ld,n,nm,nn,c1,c2,d,bvbt,ctrc,y,hom,q,p,tau,
     &                  f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,aal,
     &                  gc,go,work,qr,wa,fvec,s1)
*
* Strategy 2 for choosing the starting point for homotopy
*
* Input parameters:
* ld - leading dimension of matrices
* n,nm - dimensions of system and model
* nn - Jacobian size
* c1,c2 - parameters used in definition of D
* d,bvbt,ctrc - D, B V B^t, C^t R C
*
* Output parameters:
* y - point in vector form
* hom - parameter indicating which homotopy map will be used, depends
*       on the result of quasi-Newton iteration
*
* Working storage:
* q,p,tau,f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,aal,gc,go,work,qr
*   - working matrices
* wa,fvec,s1 - working arrays
*
      integer hom,i,ind,job,ld,n,nm,nn,z
      double precision aal(ld,ld),bvbt(ld,ld),c1,c2,ctrc(ld,ld),d(ld,ld)
      double precision eps,f10(ld,ld),f20(ld,ld),f30(ld,ld),fvec(nn)
      double precision gc(ld,ld),go(ld,ld),p(ld,ld),q(ld,ld),qr(nn,nn)
      double precision s1(ld),sigma(ld,ld),tau(ld,ld),temp1(ld,ld)
      double precision temp2(ld,ld),temp3(ld,ld),wa(nn*(nn+13)/2)
      double precision wi(ld,ld),work(ld,ld),ww(ld,ld),y(nn)
*
      job = 11
      z = ld * ld
      eps = 1.d-16
*
* Compute controllability Gramian Gc
*
      call trans(ld,d,temp2,n,n)
      call scmat(ld,-1.d0,bvbt,temp1,n,n)
      call atxpxa(temp2,aal,temp1,n,ld,ld,ld,eps,ind)
      call scmat(ld,1.d0,temp1,gc,n,n)
*
* Compute Q, the Gc rank-nm approximation
*
      call dsvdc(temp1,ld,n,n,s1,aal,temp2,ld,temp3,ld,work,job,info)
      call setmat(ld,0.d0,sigma,n,n)
      do 321 i = 1,nm
        sigma(i,i) = s1(i)
321   continue
      call mulmat(ld,temp2,sigma,temp1,n,n,n,1)
      call mulmat(ld,temp1,temp3,q,n,n,n,3)
*
* Compute observability Gramian Go
*
      call scmat(ld,1.d0,d,temp2,n,n)
      call scmat(ld,-1.d0,ctrc,temp1,n,n)
      call atxpxa(temp2,aal,temp1,n,ld,ld,ld,eps,ind)
      call scmat(ld,1.d0,temp1,go,n,n)
*
* Compute P, the Go rank-nm approximation
*
      call dsvdc(temp1,ld,n,n,s1,aal,temp2,ld,temp3,ld,work,job,info)
      call setmat(ld,0.d0,sigma,n,n)
      do 322 i = 1,nm
        sigma(i,i) = s1(i)
322   continue
      call mulmat(ld,temp2,sigma,temp1,n,n,n,1)
      call mulmat(ld,temp1,temp3,p,n,n,n,3)
*
* Compute (W,U,Sigma) for given (Q,P)
*
      call cg(ld,n,nm,q,p,ww,wi,sigma,tau,temp1,temp2,temp3,
     &        wa(1),wa(z+1),wa(2*z+1),wa(3*z+1),wa(4*z+1),wa(5*z+1),
     &        wa(6*z+1),wa(7*z+1),wa(8*z+1),wa(9*z+1),wa(10*z+1),
     &        wa(11*z+1))
*
c     write(2,231)
c231   format(/'sigma')
c     call writem(ld,sigma,nm,nm)
*
* Quasi-Newton iteration
*
*      call apprd(ld,n,nm,nn,d,bvbt,ctrc,y,hom,ww,wi,sigma,f10,f20,f30,
*     &           temp1,temp2,temp3,qr,wa,fvec,s1)
*      if (hom .ne. 1) hom = 0
*
      return
      end
