      subroutine schur(h,u,nn,nh,nu,eps,fail)
*
* Code for solving Lyapunov and Sylvester equations
* Bartels and Stewart, Comm. of the ACM, vol. 15, pp. 820-826, 1972
*
      integer nn,na,nh,fail,i,its,j,jl,k,l,ll,m,mm,m2,m3,n,nu,nm2
      logical last
      double precision h(nh,1),u(nu,1),eps,hn,rsum,test,p,q,r,s,w,x,y,z
*
      n = nn
      hn = .0
*
      do 20 i = 1,n
        jl = max0(1,i-1)
        rsum = .0
*
        do 10 j = jl,n
          rsum = rsum + dabs(h(i,j))
10      continue
        hn = dmax1(hn,rsum)
20    continue
*
      test = eps * hn
      if (hn .eq. .0) goto 230
*
30    continue
      if (n .le. 1) goto 230
      its = 0
      na = n - 1
      nm2 = n - 2
*
40    continue
      do 50 ll = 2,n
        l = n - ll + 2
        if (dabs(h(l,l-1)) .le. test) goto 60
50    continue
*
      l = 1
      goto 70
*
60    continue
      h(l,l-1) = .0
*
70    continue
      if (l .lt. na) goto 72
      n = l - 1
      goto 30
*
72    continue
      x = h(n,n) / hn
      y = h(na,na) / hn
      r = (h(n,na) / hn) * (h(na,n) / hn)
      if (its .lt. 30) goto 75
*
      fail = n
      return
*
75    continue
      if (its .eq. 10 .or. its .eq. 20) goto 80
      s = x + y
      y = x * y - r
      goto 90
*
80    continue
      y = (dabs(h(n,na)) + dabs(h(na,nm2))) / hn
      s = 1.5 * y
      y = y ** 2
*
90    continue
      its = its + 1
*
      do 100 mm = l,nm2
        m = nm2 - mm + l
        x = h(m,m) / hn
        r = h(m+1,m) / hn
        z = h(m+1,m+1) / hn
*
        p = x * (x - s) + y + r * (h(m,m+1) / hn)
        q = r * (x + z - s)
        r = r * (h(m+2,m+1) / hn)
        w = dabs(p) + dabs(q) + dabs(r)
*
        p = p / w
        q = q / w
        r = r / w
        if (m .eq. l) goto 110
        if (dabs(h(m,m-1)) * (dabs(q) + dabs(r)) .le. dabs(p) * test)
     &     goto 110
100   continue
*
110   continue
      m2 = m + 2
      m3 = m + 3
      do 120 i = m2,n
        h(i,i-2) = .0
120   continue
*
      if (m3 .gt. n) goto 140
      do 130 i = m3,n
        h(i,i-3) = .0
130   continue
*
140   continue
      do 220 k = m,na
        last = k .eq. na
        if (k .eq. m) goto 150
*
        p = h(k,k-1)
        q = h(k+1,k-1)
        r = .0
        if (.not. last) r = h(k+2,k-1)
        x = dabs(p) + dabs(q) + dabs(r)
        if (x .eq. .0) goto 220
*
        p = p / x
        q = q / x
        r = r / x
*
150     continue
        s = sqrt(p ** 2 + q ** 2 + r ** 2)
        if (p .lt. .0) s = -s
        if (k .ne. m) h(k,k-1) = -s * x
        if (k .eq. m .and. l .ne. m) h(k,k-1) = -h(k,k-1)
        p = p + s
        x = p / s
        y = q / s
        z = r / s
*
        q = q / p
        r = r / p
*
        do 170 j = k,nn
          p = h(k,j) + q * h(k+1,j)
          if (last) goto 160
          p = p + r * h(k+2,j)
          h(k+2,j) = h(k+2,j) - p * z
160       continue
          h(k+1,j) = h(k+1,j) - p * y
          h(k,j) = h(k,j) - p * x
170     continue
*
        j = min0(k+3,n)
        do 190 i = 1,j
          p = x * h(i,k) + y * h(i,k+1)
          if (last) goto 180
          p = p + z * h(i,k+2)
          h(i,k+2) = h(i,k+2) - p * r
*
180       continue
          h(i,k+1) = h(i,k+1) - p * q
          h(i,k) = h(i,k) - p
190     continue
*
        do 210 i = 1,nn
          p = x * u(i,k) + y * u(i,k+1)
          if (last) goto 200
          p = p + z * u(i,k+2)
          u(i,k+2) = u(i,k+2) - p * r
*
200       continue
          u(i,k+1) = u(i,k+1) - p * q
          u(i,k) = u(i,k) - p
210     continue
*
220   continue
      goto 40
*
230   continue
      fail = 0
*
      return
      end
