#include <stdlib.h>
#include <math.h>
#include "macro.h"
#include "eglobal.h"

void  cg(int ndim)
 
/* Compute a contragredient transformation (W,U,Sigma) of (Q,P).
Input parameters: ndim - number of rows in W
                  qt1,pt1 - (Q,P)
  iflag > 0 -- compute (W,U, Sigma) otherwise compute U, U^-1. 
 
  Output parameters:
  ttemp1,ttemp2,ttemp3 - transformation (W,U,Sigma), W = U^{-1}
  qt1,pt1 - transformation U for the input normal form ui=U^-1.
  Working storage:
  temp1--temp2,adot, - - working matrices
  er - working array                                               */ 

{ 
      int ind, k1 = 1;
      register int i,j;
      double tmp;
 
/* Compute Q = u = V * diag{D1} * V^t                                */

      tmparray(qt1,tarray1, ndim, ndim);
#if ((AIX) || (HP))
      rs(&n1,&ndim,tarray1,er,&k1,tarray2,uat,vat,&ind);
#else
      rs_(&n1,&ndim,tarray1,er,&k1,tarray2,uat,vat,&ind);
#endif

      tmpmatrix(tarray2,adot,ndim, ndim);

/*Compute ttemp1=T1 = V diag{D1^{-1/2}},ttemp2 =T1^{-1}=diag{D1^{1/2}}V^t  */
 
      setmat(0.00,temp1,ndim,ndim);
      setmat(0.00,temp2,ndim,ndim);
      for(i=0;i<ndim;i++){ 
        if(*(er+ndim-i-1)>0.0) { 
          temp1[i][i] = 1.0 / sqrt(*(er+ndim-i-1));
          temp2[i][i] = sqrt(*(er+ndim-i-1));
	}
        else {
          temp1[i][i] = 1.0;
          temp2[i][i] = 1.0;
        } 
       }
 
      for(j=0;j<ndim/2;j++){ 
         for(i=0;i<ndim;i++){ 
          tmp = adot[i][j];
          adot[i][j] = adot[i][ndim-j-1];
          adot[i][ndim-j-1] = tmp;
	  }
       } 
 
       mulmat1(adot,temp1,ttemp1,ndim,ndim,ndim);
       mulmat3(temp2,adot,ttemp2,ndim,ndim,ndim);
 
/* Compute T1^{-1} * P * T1^{-t} = P1 and store in temp2,   */
 
      mulmat1(ttemp2,pt1,temp1,ndim,ndim,ndim);
      mulmat3(temp1,ttemp2,temp2,ndim,ndim,ndim);
 
 
/* Compute P1=T2 D2 T2^t =T2 Omega T2^t,T2 =adot, T2 inverse = sigma */
 
      tmparray(temp2,tarray1,ndim, ndim);
#if ((AIX) || (HP))
      rs(&n1,&ndim,tarray1,er,&k1,tarray2,uat,vat,&ind);
#else
      rs_(&n1,&ndim,tarray1,er,&k1,tarray2,uat,vat,&ind);
#endif

      tmpmatrix(tarray2,adot,ndim,ndim);
 

      for(i=0;i<ndim/2;i++){ 
        tmp = *(er+ndim-i-1);
        *(er+ndim-i-1) = *(er + i);
        *(er+i) = tmp;
      }
 
      for(j=0;j<ndim/2;j++){ 
        for(i=0;i<ndim;i++){ 
          tmp = adot[i][j];
          adot[i][j] = adot[i][ndim-j-1];
          adot[i][ndim-j-1] = tmp;
        }
      }
 
/* Compute T2^-1                                     */
 
      trans(adot,ttemp3,ndim,ndim);
 
/* Compute U= T2^-1 * T1^-1, U^-1 = T1 * T2           */
 
      mulmat4(ttemp2,ttemp3,qt1,ndim,ndim,ndim);
      mulmat4(adot,ttemp1,pt1,ndim,ndim,ndim);
 
/* Compute Sigma, Omega^{1/4}=temp1, Omega^{-1/4} =temp2   */
 
      setmat(0.00,ttemp3,ndim,ndim);
      setmat(0.00,temp1,ndim,ndim);
      setmat(0.00,temp2,ndim,ndim);
      for(i=0;i<ndim;i++){ 
        ttemp3[i][i] = sqrt(*(er+i));
        temp1[i][i] = sqrt(ttemp3[i][i]);
        temp2[i][i] = 1.0/temp1[i][i];
       }	
 
/* Compute W = U^t * Omega^{-1/4}, W^-1 = Omega^{1/4} * U^-t */
 
      mulmat1(qt1,temp2,ttemp1,ndim,ndim,ndim);
      mulmat1(temp1,pt1,ttemp2,ndim,ndim,ndim);
 
}
