#include <stdlib.h>
#include <math.h>
#include "macro.h"
#include "eglobal.h"

void jacbm(double lambda, int k,int ll, double *y, int kk) 
/*
* JACBM -- evaluates kk-th column of Jacobian at given point
*           with respect to Bc. or w.r.t. Bc_kl 
* Input parameters: k, ll -- the index of the element of Bc
* Output parameters: y - evaluated function 
*/
{
      int ind;
      register int i,j;
      double eps = -1.0,alpha;

       setmat(0.0,att,nt,nt); /* Derivative of A tilde */
       setmat(0.0,temp1,nc,n); 
       for(i=0;i<n;i++) temp1[k][i] = cl[ll][i];
       for(i=n;i<nt;i++)      
	  for(j=0;j<n;j++) att[i][j]=temp1[i-n][j];

/* Compute derivative of Ac w.r.t. Bckl and save it in temp4 */
       alpha = - 2.0 * bc[k][ll]* ome[k][k] / bcbct[k][k];
       for(j=0;j<nc;j++){
         temp4[j][j] = - bc[k][ll] * delta[k][j];
         for(i=0;i<nc ;i++)
          if(i != j)
          temp4[i][j]= -((delta[k][j] -delta[k][i]) * alpha * ac[i][j] 
               +alpha * delta[k][j]*bcbct[i][j]*(1.0+ ome[i][i])
               +ome[j][j] * delta[k][i] *alpha * bcbct[i][j]
               +ome[j][j]*(1.0 + ome[i][i])* (delta[k][i]*bc[j][ll]
               + bc[i][ll] *delta[k][j] ) 
               - cctcc[i][j] * alpha * delta[k][j])/ (ome[j][j] -ome[i][i]);
       }

      for(i=n;i<nt;i++)
         for(j=n;j<nt;j++) att[i][j] = temp4[i-n][j-n];


       for(i=0;i<nt;i++) /* Derivative of V tilde         */
	  for(j=0;j<nt;j++) vtt[i][j]=0.0;  

       setmat(0.0,temp3,nc,nc); /* BcV2E^(ll,k)+E^(k,ll)V2Bc^T */
       for(i=0;i<nc;i++) temp3[i][k] = bcv2[i][ll];
       trans(temp3,temp2,nc,nc);
       addmat1(temp3,temp2,ttemp3,nc,nc);
       for(i=n;i<nt;i++)       /* 22 block of Vt^(j)      */ 
	  for(j=n;j<nt;j++) vtt[i][j]=ttemp3[i-n][j-n];
         
      mulmat1(att,qt,ttemp1,nt,nt,nt); /* Compute Q^(j)*/
      trans(ttemp1,ttemp2,nt,nt);
      addmat1(ttemp1,ttemp2,ttemp3,nt,nt);
      addmat4(vtt,ttemp3,ttemp1,nt,nt);
      tmparray(ttemp1,tarray2,nt,nt);

#if ((AIX) || (HP))
      atxpxa(ua,va,tarray2,&nt,&nt12,&nt12,&nt12,&eps,&ind);
#else
      atxpxa_(ua,va,tarray2,&nt,&nt12,&nt12,&nt12,&eps,&ind);
#endif

      tmpmatrix(tarray2,ttemp1,nt,nt);
      partition(ttemp1,qtd1,qtd12,qtd2,n,nt);

      mulmat1(ttemp1,rti,ttemp2,nt,nt,nt); /* Compute P^(j) */
      scmat(gaman2,ttemp2,ttemp2,nt,nt);
      addmat1(att,ttemp2,ttemp1,nt,nt);
      mulmat2(ttemp1,pt,ttemp3,nt,nt,nt);
      trans(ttemp3,ttemp1,nt,nt);
      addmat4(ttemp1,ttemp3,ttemp3,nt,nt);
      tmparray(ttemp3,tarray2,nt,nt);

#if ((AIX) || (HP))
      atxpxa(uat,vat,tarray2,&nt,&nt12,&nt12,&nt12,&eps,&ind);
#else
      atxpxa_(uat,vat,tarray2,&nt,&nt12,&nt12,&nt12,&eps,&ind);
#endif

      tmpmatrix(tarray2,ttemp1,nt,nt);
      partition(ttemp1,ptd1,ptd12,ptd2,n,nt);
 
/***************************************************/

      mulmat2(ptd12,qt12,ttemp1,nc,nc,n); 
      mulmat2(pt12,qtd12,ttemp2,nc,nc,n);  
      addmat1(ttemp1,ttemp2,ttemp1,nc,nc);

      mulmat1(ptd2,qt2,ttemp2,nc,nc,nc);
      mulmat1(pt2,qtd2,ttemp3,nc,nc,nc);  
      addmat1(ttemp2,ttemp3,ttemp2,nc,nc);

      addmat1(ttemp1,ttemp2,ttemp3,nc,nc);

      for(j=0;j<nc;j++) /* Mo^(j) in temp3        */
        for(i=j+1;i<nc;i++){
         temp3[i][j]= (ttemp3[i][j] -ttemp3[j][i]- mo[i][j]*alpha *
           (delta[j][k] -delta[i][k]) )/(ome[j][j] -ome[i][i]) ;
            temp3[j][i] = temp3[i][j];
        }

      for(i=0;i<nc;i++){
        temp3[i][i] = 0.0;
        for(j=0;j<nc;j++) 
          if(i != j)
            temp3[i][i] += (ac[i][j]+ bcbct[i][j]*ome[j][j])*temp3[j][i] 
            +(temp4[i][j]+alpha*delta[k][j]*bcbct[i][j] 
            + (bc[j][ll]*delta[k][i]+bc[i][ll]*delta[k][j])*ome[j][j])*mo[j][i];
        temp3[i][i] = -((temp4[i][i]+alpha*delta[k][i]*bcbct[i][i] 
                        + 2* bc[i][ll]*delta[k][i]*ome[i][i])*mo[i][i]
                        + temp3[i][i] ) /(ac[i][i] + bcbct[i][i]*ome[i][i]);
        }

     setmat(0.0,temp1,nc,nc);
     temp1[k][k] = alpha;

     mulmat1(temp1,mo,temp2,nc,nc,nc); /*  Mc^(j) in temp1  */
     mulmat1(ome,temp3,ttemp1,nc,nc,nc);

     addmat1(temp2,ttemp1,temp1,nc,nc);
     addmat4(temp1,ttemp3,temp1,nc,nc);

/* Compute derivatives of H_Bc */

        hbchat();
        setmat(0.0,ttemp1,nc,l);
        for(j=0;j<l;j++)
          ttemp1[k][j] = v20[ll][j] + lambda *v2dot[ll][j];
 	mulmat1(pt2,ttemp1,ttemp2,nc,l,nc);

        addmat1(ttemp2,ttemp3,ttemp1,nc,l);

        for(i=0;i<nc;i++)
          for(j=0;j<nc;j++) temp2[i][j] =mc[i][j]- ome[i][i]*mo[i][j]*ome[j][j];

        setmat(0.0, ttemp3, nc,nc);
        for(i =0;i<nc;i++) ttemp3[i][ll] = temp2[i][k];

        addmat1(ttemp3,ttemp1,ttemp1,nc,l);

        for(i=0;i<nc;i++)
          for(j=0;j<nc;j++) 
           temp2[i][j] =temp1[i][j]- ome[i][i]*temp3[i][j]*ome[j][j] 
               - alpha*delta[k][i]*mo[i][j]*ome[j][j]
               - ome[i][i]*mo[i][j]*alpha*delta[k][j] ;
        mulmat1(temp2,bc,ttemp2,nc,l,nc);

        addmat1(ttemp2,ttemp1,ttemp1,nc,l);
        toarray(nc,l,ttemp1,y);
 
/* Compute derivatives of H_Cc */
/* Original terms              */ 

      
        addmat3(temp3,temp1,ttemp1,nc,nc);
        mulmat1(cc,ttemp1,temp4,m,nc,nc);

        hcchat();  
        addmat1(ttemp3,temp4,ttemp3,m,nc);
        toarray(m,nc,ttemp3,(y+l*nc)); 
        for (i =0;i<nn;i++) y[i] = lambda * y[i];
        y[kk-2] += 1 - lambda;

} 
