function [grad] = dh2grad(k)
fprintf(1,'%c','in dh2grad');
fprintf(1,'%c \n',' ');

% DH2GRAD	computes the gradient of the Lagrangian
%		function for discrete-time H2-optimal
%		performance.  
%
%		COST = DH2GRAD(K) computes the gradients of
%		the Lagrangian for discrete-time H2-optimal 
%		performance with respect to the compensator
%		gains contained in the vector K.  DH2GRAD is 
%		a subroutine used by OPTGAIN.
%
%		SEE ALSO: OPTGAIN
%

%==================================================================
%
% REVISED: Richard S. Erwin
%          February 27, 1996
%          University of Michigan
%
%==================================================================
% load the system parameters 
%
      load sourcefile.mat

%
%  initialize variables:
%
    kstart = 1;

    nrk = kindex*indexkr';
    nck = kindex*indexkc';

    K = zeros(nrk,nck);

    for i = 1:N

          nk = indexkr(i)*indexkc(i);

          ki=k(kstart:kstart + nk - 1);
          ki=ivec(ki,indexkr(i),indexkc(i));
              
          for j = 1:kindex(i)
           
                 ijstr = [num2str(i),num2str(j)];
       
		 kstr = ['K = K + QL',ijstr,'*ki*QR',ijstr,';'];
		 eval(kstr)
		
            end;

            kstart = kstart + nk;

      end;

      invLk = inv(eye(nck) - Dyu*K);

      Atilde = A + Bu*K*invLk*Cy;
      Bwtilde = Bw + Bu*K*invLk*Dyw;
      Cztilde = Cz + Dzu*K*invLk*Cy; 

      Dzwtilde = Dzw + Dzu*K*invLk*Dyw;

      if Dzwtilde ~= zeros(size(Dzwtilde))

		error('ERROR: Closed-Loop Feed-Through Matrx Dzwtilde is NONZERO!')

      end

      Vtilde  = Bwtilde*Bwtilde';
      Rtilde  = Cztilde'*Cztilde;
      Ptilde  = dlyap(Atilde',Rtilde);
      Qtilde  = dlyap(Atilde,Vtilde);

%
% form dL/dki, convert to vector form, and append to dL/dk
%

     term1 = (eye(nrk) + Dyu'*invLk'*K');
     term2 = (Dzu'*Cztilde*Qtilde*Cy' + Bu'*Ptilde*Atilde*Qtilde*Cy' + Bu'*Ptilde*Bwtilde*Dyw');

     dLdK = term1*term2*invLk';

      for i = 1 : N

	 dLdki = zeros(indexkr(i),indexkc(i));
               
         for j = 1:kindex(i)
                
            ijstr = [num2str(i),num2str(j)];
   
	    qlstr = ['QLij = QL',ijstr,';'];
 	    qrstr = ['QRij = QR',ijstr,';'];

	    eval(qlstr)
	    eval(qrstr)

            dLdki = dLdki + 2*QLij'*dLdK*QRij';

         end

         vdLdki = vec(dLdki);

         dLdk = [dLdk;vdLdki];

     end;

     grad = dLdk;
