function [Ac,Bc,Cc,Dc,flag] = drolqg(A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw,nc)
% DROLQG 	reduced-order discrete-time lqg compensator design
%
%        	[Ac,Bc,Cc,Dc,test] = DROLQG(A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,
%		Dzw,nc) returns the ncth-order discrete-time LQG controller 
%		for system (A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw).  
%
%		TEST is a flag which indicates the closed-loop stability 
%		of the system when connected in feedback with the
%		reduced-order controller:
%
%	     		TEST = 0 := closed-loop is stable
%	     		TEST = 1 := closed-loop is unstable
%
%        	see also DLQR, DLQE, DREG, DLYAP, DBALREAL, DMODRED
%

	[Ac,Bc,Cc,Dc,temp] = dfolqg(A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw);

	np = size(Ac,1);

	elim = [nc+1:1:np];

	test = (max(abs(eig(Ac))) >= 1);

	if test == 0

		[Ac,Bc,Cc] = dbalreal(Ac,Bc,Cc);

		[Ac,Bc,Cc,Dc] = dmodred(Ac,Bc,Cc,Dc,elim);

		Dc = 0*Dc;

		[Acl,Bcl,Ccl,Dcl] = feedback(A,Bu,Cy,Dyu,Ac,Bc,Cc,Dc,1);
	
		z = eig(Acl);

		flag = (max(abs(z)) >= 1);

		if flag == 1

			disp('Closed-Loop System UNSTABLE')

		end

	else

		disp('Full-order compensator UNSTABLE -- no balanced truncation exits!')
		
		flag = 1;

	end
	