function cost = pvcost(k)

% PVCOST	evaluates the cost function for scaled Popov 
%		performance.  
%
%		COST = PVCOST(K) evaluates the cost function
%		for scaled Popov performance using the 
%		compensator gains contained in the vector K.  
%		PVCOST is a subroutine used by OPTGAIN.
%
%		SEE ALSO: OPTGAIN
%
%

% load the system parameters 
%
    load sourcefile.mat

%
%  initialize variables:
%
    kstart = 1;

    nrk = kindex*indexkr';
    nck = kindex*indexkc';

    K = zeros(nrk,nck);

    count = 1;

    for i = 1:N

          nk = indexkr(i)*indexkc(i);

          ki=k(kstart:kstart + nk - 1);
          ki=ivec(ki,indexkr(i),indexkc(i));
              
          for j = 1:kindex(i)
           
                 ijstr = [num2str(i),num2str(j)];
       
		 kstr = ['K = K + QL',ijstr,'*ki*QR',ijstr,';'];
		 eval(kstr)
		
            end;

            kstart = kstart + nk;

      end;

      invLk = inv(eye(nck) - Dyu*K);

      Atilde = A + Bu*K*invLk*Cy;
      Bwtilde = Bw + Bu*K*invLk*Dyw;
      Cztilde = Cz + Dzu*K*invLk*Cy; 

      Dzwtilde = Dzw + Dzu*K*invLk*Dyw;

      if Dzwtilde ~= zeros(size(Dzwtilde))

		error('ERROR: Closed-Loop Feed-Through Matrx Dzwtilde is NONZERO!')

      end

      Bdtilde = Bd + Bu*K*invLk*Dyd;
      Cetilde = Ce + Deu*K*invLk*Cy;

      Vtilde  = Bwtilde*Bwtilde';
      Rtilde  = Cztilde'*Cztilde;

      wvec = k(kstart:kstart+scum(nblk)-1);
      zvec = k(kstart+scum(nblk):kstart+2*scum(nblk)-1);

      WW = [];
      ZZ = [];

      for i = 1:nblk,

        if blk(i,1) == 1,

          WW = daug(WW,wvec(scum(i)));
          ZZ = daug(ZZ,zvec(scum(i)));

        elseif blk(i,1) == 2,

          WW = daug(WW,ivec(wvec((scum(i)-3):scum(i)),2,2));
          ZZ = daug(ZZ,ivec(zvec((scum(i)-3):scum(i)),2,2));

        end

      end

      invGam = inv(gam*ZZ - WW*Cetilde*Bdtilde - Bdtilde'*Cetilde'*WW);
      A0tilde   = Atilde - inv(gam)*Bdtilde*Cetilde;
      XXhat  = Cetilde'*ZZ + A0tilde'*Cetilde'*WW;

%
%  form hamiltonian for Riccatti equation:
%
%	A'X + X*A + X*R*X - Q = 0
%
%  hamiltonian is:
%
%	H = [ A     R ]
%           [ Q    -A']
%

      A = A0tilde + Bdtilde*invGam*XXhat';
      R = Bdtilde*invGam*Bdtilde';
      Q = -(Rtilde + XXhat*invGam*XXhat');

      H = [A R; Q -A'];

      [P1,P2,flag2] = ric_schr(H);
      Ptilde = P2/P1;

      cost = trace((Ptilde + 2*inv(gam)*Cetilde'*WW*Cetilde)*Vtilde);

    end
