function [A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw] = siso2tito(A,B,C,D,nu,ny)
%
% SISO2TITO	convert state space from one vector-input/one
%		vector-output to two vector-input/two vector-output.
%
%		[A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw] = siso2tito(A,B,
%		C,D,nu,ny) performs the following conversion:
%		
%			. 
%			x = A x + B | u | , | y | = C x + D | u |
%				    | w |   | z |           | w |
%
%		to
%			.
%			x = A  x + Bu  u + Bw  w
%
%			y = Cy x + Dyu u + Dyw w
%
%			z = Cz x + Dzu u + Dzw w
%
%		NU is the dimension of the actuator signal u, and
%		NY is the dimension of the measurement signal y.
%

[nout, ninp] = size(D);
np = size(A,1);

Bu  = B(:,1:nu);
Cy  = C(1:ny,:);
Dyu = D(1:ny,1:nu);

Bw  = B(:,nu+1:ninp);
Dyw = D(1:ny,nu+1:ninp);
Cz  = C(ny+1:nout,:);
Dzu = D(ny+1:nout,1:nu);
Dzw = D(ny+1:nout,nu+1:ninp);
